#include <SoftwareSerial.h>     //dodaje niezbędną bubliotekę

//dodatkowy software'owy port szeregowy dla komunikacji z Bluetooth
SoftwareSerial blueSerial(8, 9);

String baseBuffer = "";         //bufor danych z USB
String blueBuffer = "";         //bufor danych z Bluetooth
int baseComplete = 0;           //znacznik odebranych danych z USB
int blueComplete = 0;           //znacznik odebranych danych z Bluetooth
int blueStatus = 0;             //status polączenia Bluetooth, aktywe to 1
int ledPin = 13;               //dioda led na płycie Arduino


//czujnik temperatury to LM35 dołaczony do wyprowadzeń A1, A0, A2
int temperature_gnd = A0;       //masa zasilania dla LM35
int temperature_pin = A1;       //wyjście z LM35
int temperature_vdd = A2;       //plus zasilania dla LM35
int temperature_val;            //odczytanan wartość temperatury
String temperature_str;         //temperatura w postaci ramki "12,3'C"
int temperature_period = 0;     //odliczanie częstości pomiarów

//--------------------------------------------------
void setup() {
  //uruchamia sprzetowy port szeregowy dostepny z USB
  Serial.begin(19200);
  //uruchamia dodatkowy port szeregowy dla Bluetooth
  blueSerial.begin(19200);

  //konfiguruje wyprowadzenia A0 i A1 tak by zasilały czujnik temperatury
  pinMode(temperature_gnd, OUTPUT);
  pinMode(temperature_vdd, OUTPUT);
  digitalWrite(temperature_gnd, LOW);
  digitalWrite(temperature_vdd, HIGH);

  //odczyt temperatury będzie wykonywany poprzez przetwornik ADC,
  //dla wiekszej dokładności wykozystam napiecie referencyjne 1,1V
  analogReference(INTERNAL);

  //konfiguruje sterowanie diodą led
  pinMode(ledPin, OUTPUT);
  digitalWrite(ledPin, LOW);
}

//--------------------------------------------------
void loop() {
  //odczytujemy dane z portów szeregowych
  baseReceive();
  blueReceive();

  //------------------------------------------------
  if (baseComplete >= 1) {              //jeśli odebrano z USB
    blueSend(baseBuffer);               //przekaż do Bluetooth
    baseBuffer = "";                    //wyczysć bufor
    baseComplete = 0;                   //zeruj znacznik
  }

  //------------------------------------------------
  //Moduł BTM222 rozpoczyna każdy komunikat znakiem końca linii
  //więc czekamy az będa 2 znaki końca
  if (blueComplete >= 2) {              //jesli odebrano z Bluetooth
    Serial.println(blueBuffer);         //podgląd odebranych danych

    //odebranie treści "CONNECT" oznacza połączenie np ze smartfonem
    if (blueBuffer.substring(2, 9) == "CONNECT") {
      Serial.println("Polaczony!!!");   //wywyła info do USB
      blueStatus = 1;                   //ustawia aktywny status połączenia
    }

    //odebranie treści "DISCONNECT" oznacza zakończenie połaczenia
    if (blueBuffer.substring(2, 12) == "DISCONNECT") {
      Serial.println("zakonczono..."); //wysyła info do USB
      blueStatus = 0;                   //zeruje status połączenia
    }

    blueBuffer = "";                    //czyści bufor
    blueComplete = 0;                   //zeruje znacznk odebranych danych
  }

  //------------------------------------------------
  temperature_period++;                 //odicza do nastepnego pomiaru
  if (temperature_period >= 2000) {     //jeśli upłynęło ok 2s
    temperature_period = 0;             //zeruje licznik

    //odczytuje temperaturę z czujnika LM35 przy pomocy przetworika ADC
    temperature_val = analogRead(temperature_pin);
    //LM35 daje na wyjściu napięcie proporcjonalne do temperatury 10mV/*C
    //sklauje wynik i dopasowuje do napiecia referencyjnego,
    //maksymalna temp. 110*C
    temperature_val = map(temperature_val, 0, 1023, 0, 1100);

    //oddziela jedności od dziesiętnych i tworzy ramkę postaci "12,3'C"
    temperature_str = String(temperature_val / 10) + ",";
    temperature_str += String(temperature_val % 10) + "'C\r";
    Serial.println(temperature_str);    //wysyła wynik do USB

    //jeśli aktywne połaczenie to wysyła do Bluetooth
    if (blueStatus == 1) blueSend(temperature_str);
  }

  //------------------------------------------------
  if (blueStatus == 1)
    digitalWrite(ledPin, HIGH);         //aktywne połączenie LED świeci
  else
    digitalWrite(ledPin, LOW);          //brak połączenia LED nie świeci
  delay(1);
}

//--------------------------------------------------
//odbieranie danych z USB
void baseReceive()
{
  while (Serial.available()) {          //jeśli coś odebrane
    char inData = (char)Serial.read();  //odbierz dane
    baseBuffer += inData;               //i umieść w buforze
    if (inData == '\n') {               //jesli koniec linii
      baseComplete++;                 //ustaw znacznik
    }
  }
}

//--------------------------------------------------
//odbieranie danych z Bluetooth
void blueReceive()
{
  while (blueSerial.available()) {    //jeśli coś odebrane
    char inData = (char)blueSerial.read();  //odbierz dane
    blueBuffer += inData;             //i umieść w buforze
    if (inData == '\n') {             //jesli koniec linii
      blueComplete++;               //ustaw znacznik
    }
  }
}

//--------------------------------------------------
//Wysyła dane do Bluetooth
void blueSend (String data) {
  int i;
  int data_length = data.length();    //ilość danych
  //Serial.println(data);             //podgląd wysyłanych

  //komunikacja z BTM222 bez kontroli przepływu,
  //tylko sygnały RXD i TXD, bez RTS i CTS,
  //wymaga wstawienia przerwy ok 50ms po każdtym znaku
  for (i = 0; i < data_length; i++) 
  {
    //komendy dla BTM222 muszą być zakończone znakiem CR
    //dlatego zamieniam '\n' na '\r'
    if (data.charAt(i) == '\n') data.setCharAt(i, '\r');
    
    blueSerial.print(data.charAt(i)); //wywyła jeden zank
    delay(50);                        //przerwa 50ms
  }
}
